<?php

/*
  API v1.1.1
  News App - WRTeam.in
  WRTeam Developers
 */

namespace App\Controllers;

use CodeIgniter\RESTful\ResourceController;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;
use \Firebase\JWT\JWT;

class Api extends ResourceController {

    public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger) {
        // Do Not Edit This Line
        parent::initController($request, $response, $logger);
        $this->db = \Config\Database::connect();
        $this->access_key = "5670";

        $res = $this->db->table('tbl_settings')->where('type', 'jwt_key')->get()->getResult();
        $this->JWT_KEY = (!empty($res)) ? $res[0]->message : "";

        $this->helpers = helper('SystemSettings');
        date_default_timezone_set(get_system_timezone());
        $this->toDateTime = date('Y-m-d H:i:s');
    }

    public function get_live_streaming() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            $data = $this->db->table('tbl_live_streaming')->orderBy('id', 'DESC')->get()->getResult();
            if ($data) {
                $response['error'] = "false";
                $response['data'] = $data;
            } else {
                $response['error'] = "true";
                $response['message'] = "No Data Found";
            }
        }
        return $this->respond($response);
    }
    
    public function set_flag() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('comment_id') && $this->request->getVar('user_id') && $this->request->getVar('news_id') && $this->request->getVar('message')) {

                $user_id = $this->request->getVar('user_id');
                $news_id = $this->request->getVar('news_id');
                $comment_id = $this->request->getVar('comment_id');
                $message = $this->request->getVar('message');
                $status = '1';

                $data = [
                    'comment_id' => $comment_id,
                    'user_id' => $user_id,
                    'news_id' => $news_id,
                    'message' => $message,
                    'status' => $status,
                    'date' => $this->toDateTime
                ];
                $this->db->table('tbl_comment_flag')->insert($data);

                $response['error'] = "false";
                $response['message'] = "flag successfully";
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function set_like_dislike() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id') && $this->request->getVar('news_id') && $this->request->getVar('status') != '') {
                $user_id = $this->request->getVar('user_id');
                $news_id = $this->request->getVar('news_id');
                $status = $this->request->getVar('status');

                if ($status != '0') {
                    $res = $this->db->table('tbl_news_like')->where('news_id', $news_id)->where('user_id', $user_id)->get()->getResult();
                    if (!empty($res)) {
                        $data = array(
                            'status' => $status
                        );
                        $this->db->table('tbl_news_like')->where('news_id', $news_id)->where('user_id', $user_id)->update($data);
                    } else {
                        $data = [
                            'user_id' => $user_id,
                            'news_id' => $news_id,
                            'status' => $status
                        ];
                        $this->db->table('tbl_news_like')->insert($data);
                    }
                } else {
                    $this->db->table('tbl_news_like')->where('news_id', $news_id)->where('user_id', $user_id)->delete();
                }

                $response['error'] = "false";
                $response['message'] = "updated successfully!";
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function get_news_by_user_category() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id') && $this->request->getVar('category_id')) {
                $category_id = $this->request->getVar('category_id');
                $user_id = $this->request->getVar('user_id');

                $offset = ($this->request->getVar('offset')) ? $this->request->getVar('offset') : 0;
                $limit = ($this->request->getVar('limit')) ? $this->request->getVar('limit') : 10;

                $data = $this->db->query("SELECT n.*,c.category_name FROM tbl_news n JOIN tbl_category c ON n.category_id = c.id WHERE n.category_id IN ($category_id)  ORDER BY rand() LIMIT $offset, $limit")->getResult();

                if ($data) {
                    $data1 = $this->db->query("SELECT n.*,c.category_name FROM tbl_news n JOIN tbl_category c ON n.category_id = c.id WHERE n.category_id IN ($category_id)")->getResult();
                    $total = count($data1);
                    for ($i = 0; $i < count($data); $i++) {
                        $data[$i]->image = ($data[$i]->image) ? base_url() . '/public/images/news/' . $data[$i]->image : '';
                        if ($data[$i]->content_type == "video_upload") {
                            $data[$i]->content_value = ($data[$i]->content_value) ? base_url() . '/public/images/news_video/' . $data[$i]->content_value : '';
                        }
                        $img = array();
                        $img = $this->db->table('tbl_news_image')->select('other_image')->where('news_id', $data[$i]->id)->get()->getResult();
                        for ($j = 0; $j < count($img); $j++) {
                            $img[$j]->other_image = ($img[$j]->other_image) ? base_url() . '/public/images/news/' . $data[$i]->id . '/' . $img[$j]->other_image : '';
                        }
                        $data[$i]->image_data = $img;

                        $like = $this->db->table('tbl_news_like')->select('COUNT(id) as total')->where('news_id', $data[$i]->id)->where('status', '1')->get()->getResult();
                        $data[$i]->total_like = (!empty($like)) ? $like[0]->total : "0";

                        $dislike = $this->db->table('tbl_news_like')->select('COUNT(id) as total')->where('news_id', $data[$i]->id)->where('status', '2')->get()->getResult();
                        $data[$i]->total_dislike = (!empty($dislike)) ? $dislike[0]->total : "0";

                        $ulike = $this->db->table('tbl_news_like')->where('news_id', $data[$i]->id)->where('user_id', $user_id)->where('status', '1')->get()->getResult();
                        $data[$i]->like = (!empty($ulike)) ? '1' : '0';

                        $udislike = $this->db->table('tbl_news_like')->where('news_id', $data[$i]->id)->where('user_id', $user_id)->where('status', '2')->get()->getResult();
                        $data[$i]->dislike = (!empty($udislike)) ? '1' : '0';
                    }
                    $response['error'] = "false";
                    $response['total'] = "$total";
                    $response['data'] = $data;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "No Data Found";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function set_user_category() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id') && $this->request->getVar('category_id') != '') {
                $user_id = $this->request->getVar('user_id');
                $category_id = $this->request->getVar('category_id');
                if ($category_id == '0') {
                    $this->db->table('tbl_users_category')->where('user_id', $user_id)->delete();
                } else {
                    $res = $this->db->table('tbl_users_category')->where('user_id', $user_id)->get()->getResult();
                    if (!empty($res)) {
                        $data = ['category_id' => $category_id];
                        $this->db->table('tbl_users_category')->where('user_id', $user_id)->update($data);
                    } else {
                        $data = [
                            'user_id' => $user_id,
                            'category_id' => $category_id,
                        ];
                        $this->db->table('tbl_users_category')->insert($data);
                    }
                }
                $response['error'] = "false";
                $response['message'] = "updated successfully";
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function get_notification() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            $offset = ($this->request->getVar('offset')) ? $this->request->getVar('offset') : 0;
            $limit = ($this->request->getVar('limit')) ? $this->request->getVar('limit') : 10;

            $res = $this->db->table('tbl_notifications');
            $res->select('tbl_notifications.*,tbl_category.category_name, tbl_news.title as new_title');
            $res->join('tbl_category', 'tbl_category.id = tbl_notifications.category_id');
            $res->join('tbl_news', 'tbl_news.id = tbl_notifications.news_id');
            $res->limit($limit, $offset)->orderBy('tbl_notifications.id', 'DESC');
            $data = $res->get()->getResult();
            if ($data) {
                $res = $this->db->table('tbl_notifications');
                $res->select('tbl_notifications.*,tbl_category.category_name');
                $res->join('tbl_category', 'tbl_category.id = tbl_notifications.category_id');
                $res->join('tbl_news', 'tbl_news.id = tbl_notifications.news_id');
                $data1 = $res->get()->getResult();
                $total = count($data1);
                for ($i = 0; $i < count($data); $i++) {
                    $data[$i]->image = ($data[$i]->image) ? base_url() . '/public/images/notification/' . $data[$i]->image : '';
                }
                $response['error'] = "false";
                $response['total'] = "$total";
                $response['data'] = $data;
            } else {
                $response['error'] = "true";
                $response['message'] = "No Data Found";
            }
        }
        return $this->respond($response);
    }

    public function get_bookmark() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id')) {
                $user_id = $this->request->getVar('user_id');

                $offset = ($this->request->getVar('offset')) ? $this->request->getVar('offset') : 0;
                $limit = ($this->request->getVar('limit')) ? $this->request->getVar('limit') : 10;

                $res = $this->db->table('tbl_bookmark b');
                $res->select('b.*,n.category_id,c.category_name,n.title,n.date,n.content_type,n.content_value,n.image,n.description');
                $res->join('tbl_news n', 'n.id = b.news_id');
                $res->join('tbl_category c', 'c.id = n.category_id');
                $res->where('user_id', $user_id)->limit($limit, $offset)->orderBy('id', 'DESC');
                $data = $res->get()->getResult();
                if ($data) {
                    $res = $this->db->table('tbl_bookmark b');
                    $res->select('b.*,n.category_id,c.category_name,n.title,n.date,n.content_type,n.content_value,n.image,n.description');
                    $res->join('tbl_news n', 'n.id = b.news_id');
                    $res->join('tbl_category c', 'c.id = n.category_id');
                    $res->where('user_id', $user_id);
                    $data1 = $res->get()->getResult();
                    $total = count($data1);

                    for ($i = 0; $i < count($data); $i++) {
                        $data[$i]->image = ($data[$i]->image) ? base_url() . '/public/images/news/' . $data[$i]->image : '';
                        if ($data[$i]->content_type == "video_upload") {
                            $data[$i]->content_value = ($data[$i]->content_value) ? base_url() . '/public/images/news_video/' . $data[$i]->content_value : '';
                        }
                        $img = array();
                        $img = $this->db->table('tbl_news_image')->select('other_image')->where('news_id', $data[$i]->news_id)->get()->getResult();
                        for ($j = 0; $j < count($img); $j++) {
                            $img[$j]->other_image = ($img[$j]->other_image) ? base_url() . '/public/images/news/' . $data[$i]->news_id . '/' . $img[$j]->other_image : '';
                        }
                        $data[$i]->image_data = $img;

                        $like = $this->db->table('tbl_news_like')->select('COUNT(id) as total')->where('news_id', $data[$i]->news_id)->where('status', '1')->get()->getResult();
                        $data[$i]->total_like = (!empty($like)) ? $like[0]->total : "0";

                        $dislike = $this->db->table('tbl_news_like')->select('COUNT(id) as total')->where('news_id', $data[$i]->news_id)->where('status', '2')->get()->getResult();
                        $data[$i]->total_dislike = (!empty($dislike)) ? $dislike[0]->total : "0";
                        
                        $ulike = $this->db->table('tbl_news_like')->where('news_id', $data[$i]->news_id)->where('user_id', $user_id)->where('status', '1')->get()->getResult();
                        $data[$i]->like = (!empty($ulike)) ? '1' : '0';

                        $udislike = $this->db->table('tbl_news_like')->where('news_id', $data[$i]->news_id)->where('user_id', $user_id)->where('status', '2')->get()->getResult();
                        $data[$i]->dislike = (!empty($udislike)) ? '1' : '0';
                    }
                    $response['error'] = "false";
                    $response['total'] = "$total";
                    $response['data'] = $data;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "No Data Found";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function set_bookmark() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id') && $this->request->getVar('news_id') && $this->request->getVar('status') != '') {
                $user_id = $this->request->getVar('user_id');
                $news_id = $this->request->getVar('news_id');
                $status = $this->request->getVar('status');

                if ($status == '1') {
                    $data = $this->db->table('tbl_bookmark')->where('user_id', $user_id)->where('news_id', $news_id)->get()->getResult();
                    if (empty($data)) {
                        $data = [
                            'user_id' => $user_id,
                            'news_id' => $news_id,
                        ];
                        $this->db->table('tbl_bookmark')->insert($data);
                        $response['error'] = "false";
                        $response['message'] = "bookmark successfully";
                    } else {
                        $response['error'] = "true";
                        $response['message'] = "already bookmark";
                    }
                } else if ($status == '0') {
                    $this->db->table('tbl_bookmark')->where('user_id', $user_id)->where('news_id', $news_id)->delete();
                    $response['error'] = "false";
                    $response['message'] = "bookmark remove successfully";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function delete_comment() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id') && $this->request->getVar('comment_id')) {
                $user_id = $this->request->getVar('user_id');
                $comment_id = $this->request->getVar('comment_id');

                $this->db->table('tbl_comment')->where('id', $comment_id)->where('user_id', $user_id)->delete();
                $response['error'] = "false";
                $response['message'] = "comment deleted!";
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function get_comment_by_news() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('news_id')) {
                $news_id = $this->request->getVar('news_id');

                $offset = ($this->request->getVar('offset')) ? $this->request->getVar('offset') : 0;
                $limit = ($this->request->getVar('limit')) ? $this->request->getVar('limit') : 10;

                $res = $this->db->table('tbl_comment c');
                $res->select('c.*,u.name,u.profile');
                $res->join('tbl_users u', 'u.id = c.user_id');
                $res->where('c.news_id', $news_id)->where('c.status', '1')->limit($limit, $offset)->orderBy('id', 'DESC');
                $data = $res->get()->getResult();
                if ($data) {
                    for ($i = 0; $i < count($data); $i++) {
                        if (filter_var($data[$i]->profile, FILTER_VALIDATE_URL) === FALSE) {
                            // Not a valid URL. Its a image only or empty
                            $data[$i]->profile = (!empty($data[$i]->profile)) ? base_url() . '/public/images/profile/' . $data[$i]->profile : '';
                        }
                    }
                    $res = $this->db->table('tbl_comment c');
                    $res->select('c.*, u.name')->join('tbl_users u', 'u.id = c.user_id');
                    $res->where('c.news_id', $news_id)->where('c.status', '1');
                    $data1 = $res->get()->getResult();
                    $total = count($data1);

                    $response['error'] = "false";
                    $response['total'] = "$total";
                    $response['data'] = $data;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "No Data Found";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function set_comment() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id') && $this->request->getVar('news_id') && $this->request->getVar('message')) {
                $user_id = $this->request->getVar('user_id');
                $news_id = $this->request->getVar('news_id');
                $message = $this->request->getVar('message');
                $status = '1';

                $data2 = [
                    'user_id' => $user_id,
                    'news_id' => $news_id,
                    'message' => $message,
                    'status' => $status,
                    'date' => $this->toDateTime
                ];
                $this->db->table('tbl_comment')->insert($data2);

                $offset = ($this->request->getVar('offset')) ? $this->request->getVar('offset') : 0;
                $limit = ($this->request->getVar('limit')) ? $this->request->getVar('limit') : 10;

                $res = $this->db->table('tbl_comment c');
                $res->select('c.*,u.name,u.profile');
                $res->join('tbl_users u', 'u.id = c.user_id');
                $res->where('c.news_id', $news_id)->where('c.status', '1')->limit($limit, $offset)->orderBy('id', 'DESC');
                $data = $res->get()->getResult();
                if ($data) {
                    for ($i = 0; $i < count($data); $i++) {
                        if (filter_var($data[$i]->profile, FILTER_VALIDATE_URL) === FALSE) {
                            // Not a valid URL. Its a image only or empty
                            $data[$i]->profile = (!empty($data[$i]->profile)) ? base_url() . '/public/images/profile/' . $data[$i]->profile : '';
                        }
                    }
                    $res = $this->db->table('tbl_comment c');
                    $res->select('c.*, u.name')->join('tbl_users u', 'u.id = c.user_id');
                    $res->where('c.news_id', $news_id)->where('c.status', '1');
                    $data1 = $res->get()->getResult();
                    $total = count($data1);

                    $response['error'] = "false";
                    $response['message'] = "Comment successfully";
                    $response['total'] = "$total";
                    $response['data'] = $data;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "No Data Found";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function get_settings() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            $data = $this->db->table('tbl_settings')->get()->getResult();
            if ($data) {
                for ($i = 0; $i < count($data); $i++) {
                    $data1[$data[$i]->type] = $data[$i]->message;
                }
                $response['error'] = "false";
                $response['data'] = $data1;
            } else {
                $response['error'] = "true";
                $response['message'] = "No data found!";
            }
        }
        return $this->respond($response);
    }

    public function get_user_by_id() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id')) {
                $user_id = $this->request->getVar('user_id');

                $res = $this->db->table('tbl_users')->where('id', $user_id)->get()->getResult();
                if ($res) {
                    for ($i = 0; $i < count($res); $i++) {
                        if (filter_var($res[$i]->profile, FILTER_VALIDATE_URL) === FALSE) {
                            // Not a valid URL. Its a image only or empty
                            $res[$i]->profile = (!empty($res[$i]->profile)) ? base_url() . '/public/images/profile/' . $res[$i]->profile : '';
                        }
                        $cat = $this->db->table('tbl_users_category')->select('category_id')->where('user_id', $user_id)->get()->getResult();
                        $res[$i]->category_id = ($cat) ? $cat[0]->category_id : '';
                    }
                    $response['error'] = "false";
                    $response['data'] = $res;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "No Data Found";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function register_token() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('token')) {
                $token = $this->request->getVar('token');
                $data = $this->db->table('tbl_token')->where('token', $token)->get()->getResult();
                if (empty($data)) {
                    $data = ['token' => $token];
                    $this->db->table('tbl_token')->insert($data);
                    $response['error'] = "false";
                    $response['message'] = "Device registered successfully";
                } else {
                    $response['error'] = "true";
                    $response['message'] = "Device already registered";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function set_profile_image() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            $image = $this->request->getFile('image');
            if ($this->request->getVar('user_id') && $image->getClientName() != '') {
                if (!is_dir('public/images/profile')) {
                    mkdir('./public/images/profile', 0777, TRUE);
                }
                $user_id = $this->request->getVar('user_id');
                $data = $this->db->table('tbl_users')->where('id', $user_id)->get()->getResult();
                if ($data[0]->profile != "") {
                    if (file_exists('public/images/profile/' . $data[0]->profile)) {
                        unlink('public/images/profile/' . $data[0]->profile);
                    }
                }
                $newName = microtime(TRUE) . '.' . $image->getClientExtension();
                if ($image->isValid() && !$image->hasMoved()) {
                    $image->move('public/images/profile', $newName);
                }
                if ($image->hasMoved()) {
                    $data = ['profile' => $newName];
                    $this->db->table('tbl_users')->where('id', $user_id)->update($data);
                    $response['error'] = "false";
                    $response['message'] = 'File uploaded successfully!';
                    $response['file_path'] = base_url() . '/public/images/profile/' . $newName;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "Could not move the file!";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function update_profile() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id')) {
                $user_id = $this->request->getVar('user_id');

                $name = ($this->request->getVar('name')) ? $this->request->getVar('name') : "";
                $mobile = ($this->request->getVar('mobile')) ? $this->request->getVar('mobile') : "";

                $data = array(
                    'name' => $name,
                    'mobile' => $mobile
                );
                $this->db->table('tbl_users')->where('id', $user_id)->update($data);

                $response['error'] = "false";
                $response['message'] = "Profile updated successfully";
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function user_signup() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('firebase_id') && $type = $this->request->getVar('type')) {
                $firebase_id = $this->request->getVar('firebase_id');
                $email = ($this->request->getVar('email')) ? $this->request->getVar('email') : "";
                $type = $this->request->getVar('type');
                $name = ($this->request->getVar('name')) ? $this->request->getVar('name') : "";
                $mobile = ($this->request->getVar('mobile')) ? $this->request->getVar('mobile') : "";
                $status = ($this->request->getVar('status')) ? $this->request->getVar('status') : "1";
                $profile = ($this->request->getVar('profile')) ? $this->request->getVar('profile') : "";

                $res = $this->db->table('tbl_users')->where('firebase_id', $firebase_id)->get()->getResult();

                if (empty($res)) {
                    $data = [
                        'firebase_id' => $firebase_id,
                        'name' => $name,
                        'email' => $email,
                        'mobile' => $mobile,
                        'type' => $type,
                        'profile' => $profile,
                        'status' => $status,
                        'date' => $this->toDateTime
                    ];
                    $this->db->table('tbl_users')->insert($data);
                    $insert_id = $this->db->insertID();
                    $data1 = $this->db->table('tbl_users')->where('id', $insert_id)->get()->getResult();
                    $data1[0]->is_login = '1';
                    $response['error'] = "false";
                    $response['message'] = "User Registered successfully";
                    $response['data'] = $data1[0];
                } else {
                    for ($i = 0; $i < count($res); $i++) {
                        if (filter_var($res[$i]->profile, FILTER_VALIDATE_URL) === FALSE) {
                            // Not a valid URL. Its a image only or empty
                            $res[$i]->profile = (!empty($res[$i]->profile)) ? base_url() . '/public/images/profile/' . $res[$i]->profile : '';
                        }
                        $res[$i]->is_login = '0';
                    }
                    $response['error'] = "false";
                    $response['message'] = "Successfully logged in";
                    $response['data'] = $res[0];
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function get_news_by_id() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('news_id')) {
                $news_id = $this->request->getVar('news_id');

                $res = $this->db->table('tbl_news');
                $res->select('tbl_news.*, tbl_category.category_name');
                $res->join('tbl_category', 'tbl_category.id = tbl_news.category_id');
                $res->where('tbl_news.id', $news_id);
                $data = $res->get()->getResult();
                if ($data) {
                    for ($i = 0; $i < count($data); $i++) {
                        $data[$i]->image = ($data[$i]->image) ? base_url() . '/public/images/news/' . $data[$i]->image : '';
                        if ($data[$i]->content_type == "video_upload") {
                            $data[$i]->content_value = ($data[$i]->content_value) ? base_url() . '/public/images/news_video/' . $data[$i]->content_value : '';
                        }
                    }
                    $response['error'] = "false";
                    $response['data'] = $data;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "No Data Found";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function get_news_by_category() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('category_id') && $this->request->getVar('user_id') != '') {
                $category_id = $this->request->getVar('category_id');
                $user_id = $this->request->getVar('user_id');

                $offset = ($this->request->getVar('offset')) ? $this->request->getVar('offset') : 0;
                $limit = ($this->request->getVar('limit')) ? $this->request->getVar('limit') : 10;

                $res = $this->db->table('tbl_news');
                $res->select('tbl_news.*,tbl_category.category_name');
                $res->join('tbl_category', 'tbl_news.category_id = tbl_category.id');
                $res->where('tbl_news.category_id', $category_id)->limit($limit, $offset)->orderBy('id', 'DESC');
                $data = $res->get()->getResult();
                if ($data) {
                    $data1 = $this->db->table('tbl_news')->join('tbl_category', 'tbl_news.category_id = tbl_category.id')->where('tbl_news.category_id', $category_id)->get()->getResult();
                    $total = count($data1);
                    for ($i = 0; $i < count($data); $i++) {
                        $data[$i]->image = ($data[$i]->image) ? base_url() . '/public/images/news/' . $data[$i]->image : '';
                        if ($data[$i]->content_type == "video_upload") {
                            $data[$i]->content_value = ($data[$i]->content_value) ? base_url() . '/public/images/news_video/' . $data[$i]->content_value : '';
                        }
                        $img = array();
                        $img = $this->db->table('tbl_news_image')->select('other_image')->where('news_id', $data[$i]->id)->get()->getResult();
                        for ($j = 0; $j < count($img); $j++) {
                            $img[$j]->other_image = ($img[$j]->other_image) ? base_url() . '/public/images/news/' . $data[$i]->id . '/' . $img[$j]->other_image : '';
                        }
                        $data[$i]->image_data = $img;

                        $like = $this->db->table('tbl_news_like')->select('COUNT(id) as total')->where('news_id', $data[$i]->id)->where('status', '1')->get()->getResult();
                        $data[$i]->total_like = (!empty($like)) ? $like[0]->total : "0";

                        $dislike = $this->db->table('tbl_news_like')->select('COUNT(id) as total')->where('news_id', $data[$i]->id)->where('status', '2')->get()->getResult();
                        $data[$i]->total_dislike = (!empty($dislike)) ? $dislike[0]->total : "0";

                        $ulike = $this->db->table('tbl_news_like')->where('news_id', $data[$i]->id)->where('user_id', $user_id)->where('status', '1')->get()->getResult();
                        $data[$i]->like = (!empty($ulike)) ? '1' : '0';

                        $udislike = $this->db->table('tbl_news_like')->where('news_id', $data[$i]->id)->where('user_id', $user_id)->where('status', '2')->get()->getResult();
                        $data[$i]->dislike = (!empty($udislike)) ? '1' : '0';
                    }
                    $response['error'] = "false";
                    $response['total'] = "$total";
                    $response['data'] = $data;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "No Data Found";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function get_news() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            if ($this->request->getVar('user_id') != '') {
                $user_id = $this->request->getVar('user_id');
                $offset = ($this->request->getVar('offset')) ? $this->request->getVar('offset') : 0;
                $limit = ($this->request->getVar('limit')) ? $this->request->getVar('limit') : 10;

                $res = $this->db->table('tbl_news');
                $res->select('tbl_news.*,tbl_category.category_name');
                $res->join('tbl_category', 'tbl_category.id = tbl_news.category_id', 'left');
                $res->limit($limit, $offset)->orderBy('id', 'DESC');
                $data = $res->get()->getResult();
                if ($data) {
                    $res = $this->db->table('tbl_news');
                    $res->select('tbl_news.*,tbl_category.category_name');
                    $res->join('tbl_category', 'tbl_category.id = tbl_news.category_id', 'left');
                    $data1 = $res->get()->getResult();
                    $total = count($data1);
                    for ($i = 0; $i < count($data); $i++) {
                        $data[$i]->image = ($data[$i]->image) ? base_url() . '/public/images/news/' . $data[$i]->image : '';
                        if ($data[$i]->content_type == "video_upload") {
                            $data[$i]->content_value = ($data[$i]->content_value) ? base_url() . '/public/images/news_video/' . $data[$i]->content_value : '';
                        }
                        $img = array();
                        $img = $this->db->table('tbl_news_image')->select('other_image')->where('news_id', $data[$i]->id)->get()->getResult();
                        for ($j = 0; $j < count($img); $j++) {
                            $img[$j]->other_image = ($img[$j]->other_image) ? base_url() . '/public/images/news/' . $data[$i]->id . '/' . $img[$j]->other_image : '';
                        }
                        $data[$i]->image_data = $img;

                        $like = $this->db->table('tbl_news_like')->select('COUNT(id) as total')->where('news_id', $data[$i]->id)->where('status', '1')->get()->getResult();
                        $data[$i]->total_like = (!empty($like)) ? $like[0]->total : "0";

                        $dislike = $this->db->table('tbl_news_like')->select('COUNT(id) as total')->where('news_id', $data[$i]->id)->where('status', '2')->get()->getResult();
                        $data[$i]->total_dislike = (!empty($dislike)) ? $dislike[0]->total : "0";

                        $ulike = $this->db->table('tbl_news_like')->where('news_id', $data[$i]->id)->where('user_id', $user_id)->where('status', '1')->get()->getResult();
                        $data[$i]->like = (!empty($ulike)) ? '1' : '0';

                        $udislike = $this->db->table('tbl_news_like')->where('news_id', $data[$i]->id)->where('user_id', $user_id)->where('status', '2')->get()->getResult();
                        $data[$i]->dislike = (!empty($udislike)) ? '1' : '0';
                    }
                    $response['error'] = "false";
                    $response['total'] = "$total";
                    $response['data'] = $data;
                } else {
                    $response['error'] = "true";
                    $response['message'] = "No Data Found";
                }
            } else {
                $response['error'] = "true";
                $response['message'] = "Please fill all the data and submit!";
            }
        }
        return $this->respond($response);
    }

    public function get_category() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            $offset = ($this->request->getVar('offset')) ? $this->request->getVar('offset') : 0;
            $limit = ($this->request->getVar('limit')) ? $this->request->getVar('limit') : 10;

            $data = $this->db->table('tbl_category')->limit($limit, $offset)->orderBy('id', 'DESC')->get()->getResult();
            if ($data) {
                $data1 = $this->db->table('tbl_category')->get()->getResult();
                $total = count($data1);
                for ($i = 0; $i < count($data); $i++) {
                    $data[$i]->image = ($data[$i]->image) ? base_url() . '/public/images/category/' . $data[$i]->image : '';
                }
                $response['error'] = "false";
                $response['total'] = "$total";
                $response['data'] = $data;
            } else {
                $response['error'] = "true";
                $response['message'] = "No Data Found";
            }
        }
        return $this->respond($response);
    }

    public function get_breaking_news() {
        if ($this->verify_token()) {
            return $this->verify_token();
        }
        if ($this->access_key != $this->request->getVar('access_key')) {
            $response['error'] = "true";
            $response['message'] = "Invalid Access Key";
        } else {
            $data = $this->db->table('tbl_breaking_news')->orderBy('id', 'DESC')->get()->getResult();
            if ($data) {
                $response['error'] = "false";
                $response['data'] = $data;
            } else {
                $response['error'] = "true";
                $response['message'] = "No Data Found";
            }
        }
        return $this->respond($response);
    }

    public function generate_token() {
        $payload = [
            'iat' => time(), /* issued at time */
            'iss' => 'NewsAPP',
            'exp' => time() + (30 * 60 * 60 * 24), /* expires after 1 minute */
            'sub' => 'News APP Authentication'
        ];
        return JWT::encode($payload, $this->JWT_KEY);
    }

    public function verify_token() {
        try {
            $token = JWT::getBearerToken();
        } catch (\Exception $e) {
            $response['error'] = "true";
            $response['message'] = $e->getMessage();
            return $this->respond($response);
        }
        if (!empty($token)) {
            try {
                $payload = JWT::decode($token, $this->JWT_KEY, ['HS256']);
                if (!isset($payload->iss) || $payload->iss != 'NewsAPP') {
                    $response['error'] = "true";
                    $response['message'] = 'Invalid Hash';
                    return $this->respond($response);
                }
            } catch (\Exception $e) {
                $response['error'] = "true";
                $response['message'] = $e->getMessage();
                return $this->respond($response);
            }
        } else {
            $response['error'] = "true";
            $response['message'] = "Unauthorized access not allowed";
            return $this->respond($response);
        }
    }

}
